//+------------------------------------------------------------------+
//|                                             AvoidTheNewsV0_1.mqh |
//|                                               Paul Hampton-Smith |
//|                                               paul1000@pobox.com |
//+------------------------------------------------------------------+
/*


This include file is intended to be used with systems that work on the shorter timeframes (1M, 5M and 15M) to 
give them the opportunity to be out of the market during news events.

The main function is 
   bool NewsEvent(int nWithinMinutes, string& strDescription, datetime& NewsTimeServer)

which can be used as follows

   #include <AvoidTheNewsVn_n.mqh>

   string strNews; 
   datetime NewsTime;

   // avoid buying if within 5 minutes of news event
   if (MyBuySignal && !NewsEvent(5,strNews,NewsTime)) 
      DoMyBuying();

   // close all positions if within 5 minutes of news event
   if (OpenOrders() && NewsEvent(5,strNews,NewsTime)) 
      CloseOrDeleteAllOrders();


THE FOLLOWING TWO ITEMS NEED TO BE SET UP
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

1. Global variable UTCtoServerTimeInMinutes is added to each instance of MT4 (see menu item Files->Global Variables)

It enables different instances of MT4 using different brokers to run EAs with identical time triggers

Examples:
   UTCtoServerTimeInMinutes = 0   for InterbankFX because their server time = UTC
   UTCtoServerTimeInMinutes = 180 for FXDD        because their server time = UTC plus 3 hours



2. File AvoidTheNews.txt is added to the experts\files folder

Format of the file is as per the following example (the header must be included)

Description,Currency,DateTime,TimeZone
News description 1,USD,2006.08.09 12:00,UTC
News description 2,GBP,2006.08.09 17:30,UTC
News occurring every day in US,USD,08:30,EST
News occurring every day in GB,GBP,09:30,BST

Valid values for each of the fields are

1. Description: string without commas

2. Currency: any 3 alpha currency string

3. DateTime: yyyy.mm.dd hh:mm  for a specific date 
     or      hh:mm  for a time that occurs every Day
     this second option can be a convenient way of ensuring that frequently used news times are always defined 
     without having to update the file

4. TimeZone
   UTC: Time is in Greenwich Mean Time
   BST: Time is in British Summer Time (UTC + 1 hour = eg London with daylight saving)
   EST: Time is in Eastern Standard Time (UTC - 5 hours - eg New York)
   EDT: Time is in Eastern Daylight Time (UTC - 4 hours = eg New York with daylight saving)


*////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////


#property copyright "Paul Hampton-Smith"
#property link      ""

#include <stdlib.mqh>
#include <stderror.mqh>

string strNewsFileName = "AvoidTheNews.txt"; // located in experts\files
int chNewsFileSeparator = ','; 

bool bDebug = false; // switch on for lots more Print messages


//////////////////////////////////////////////
// MAIN FUNCTION
//////////////////////////////////////////////

bool NewsEvent(int nWithinMinutes, string& strDescription, datetime& NewsTimeServer)
{
   NextNews(strDescription, NewsTimeServer, nWithinMinutes);

   bool bReturn = CurTime() >= NewsTimeServer - nWithinMinutes*60 && 
               CurTime() <= NewsTimeServer + nWithinMinutes*60;
   if (bDebug) Print("NewsEvent(",nWithinMinutes,") returning ",bToStr(bReturn)," and (''",strDescription,"'',",TimeToStr(NewsTimeServer),")");
   return(bReturn);
}

//////////////////////////////////////////////
//////////////////////////////////////////////


bool NextNews(string& strDescription, datetime& EarliestNextTime, int nMinutesInThePast = 0)
{
   bool bReturn;
   static bool bDontBugMe = false;

   int handle = FileOpen(strNewsFileName, FILE_CSV|FILE_READ, chNewsFileSeparator);
   if (handle <= 0)
   {   
      // only Alert the user once to the fact that this variable is missing
      if (!bDontBugMe)
      {
         Alert("Please add file ",strNewsFileName," to experts\files folder, see AvoidTheNews.mqh for format");
         bDontBugMe = true;
      }
      // repeated entries into log
      Print("Please add file ",strNewsFileName," to experts\files folder, see AvoidTheNews.mqh for format");
      return(0);
   }

   string strCurrency;
   datetime VeryFutureTime = StrToTime("2030.01.01 00:00");
   EarliestNextTime = VeryFutureTime;
   datetime DateTimeServer;

   // read header line
   while(!FileIsLineEnding(handle)) FileReadString(handle);
 
   // read news date lines
   while (!FileIsEnding(handle))
   {
      string strDescr, strCurr; datetime dtServer;
      if (!ReadCSVline(handle, strDescr, strCurr, dtServer)) break;
      strCurrency = strCurr; DateTimeServer = dtServer;
      if (CurTime()-nMinutesInThePast*60 > DateTimeServer || !CurrencyPairContains(strCurrency)) continue;
      if (EarliestNextTime > DateTimeServer)
      {
         EarliestNextTime = DateTimeServer;
         strDescription = strDescr;
      }
   }
   FileClose(handle);
   if (EarliestNextTime == VeryFutureTime)  
   {
      // No dates in the future
      EarliestNextTime = 0;
      DateTimeServer = 0;
      strDescription = "";
      bReturn = false;
   }
   
   if (bDebug) Print("Function NextNews() returning ",TimeToStr(EarliestNextTime)," and (''",strDescription,"'')");
   return(bReturn);
}  

datetime ServerTime(datetime UTC)
{
   // Uses global variable UTCtoServerTimeInMinutes to convert UTC to server time.
   // this enables different instances of MT4 using different brokers to run EAs with identical UTC time triggers
   // Examples:
   //    UTCtoServerTimeInMinutes = 0   for InterbankFX because their server time = UTC
   //    UTCtoServerTimeInMinutes = 180 for FXDD        because their server time = UTC plus 3 hours

   static bool bDontBugMe = false;
   if (!GlobalVariableCheck("UTCtoServerTimeInMinutes"))
   {
      // only Alert the user once to the fact that this variable is missing
      if (!bDontBugMe)
      {
         Alert("Please add global variable UTCtoServerTimeInMinutes (menu item Tools->Global Variables)");
         bDontBugMe = true;
      }
      // repeated entries into log
      Print("Please add global variable UTCtoServerTimeInMinutes (menu item Tools->Global Variables)");
      return(0);
   }
   if (bDebug) Print("ServerTime(",TimeToStr(UTC),") returning ",TimeToStr(UTC+60*GlobalVariableGet("UTCtoServerTimeInMinutes")));
   return(UTC+60*GlobalVariableGet("UTCtoServerTimeInMinutes"));
}

datetime TimeToUTC(datetime dt, string strTimeZone)
{
   int BST_TimeZone = 1;
   int EST_TimeZone = -4;
   int EDT_TimeZone = -5;

   datetime dtReturn;
   if      (strTimeZone == "UTC") dtReturn = dt;
   else if (strTimeZone == "BST") dtReturn = dt - BST_TimeZone*3600;
   else if (strTimeZone == "EST") dtReturn = dt - EST_TimeZone*3600;
   else if (strTimeZone == "EDT") dtReturn = dt - EDT_TimeZone*3600;
   else 
   { 
      Print("Unknown timezone ",strTimeZone, " in TimeToUTC()");
      dtReturn = 0;
   }
   
   if (bDebug) Print("TimeToUTC(",TimeToStr(dt),",",strTimeZone,") returning ",TimeToStr(dtReturn));
   return(dtReturn);
}      

bool ReadCSVline(int handle, string& strDescription, string& strCurrency, datetime& DateTimeServer)
{
   int nLastError = 0;

//////////////////////////////////////
   // Handle

   if (handle <= 0)
   {
      Print("Invalid file handle for news file", strNewsFileName);
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // Description
   strDescription = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      if (nLastError != ERR_END_OF_FILE) Print(ErrorDescription(nLastError)," reading description from ",strNewsFileName);
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // Currency
//   tmp = FileReadString(handle); nLastError = GetLastError();
   strCurrency = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      Print(ErrorDescription(nLastError)," reading currency from ",strNewsFileName);
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }
   if (StringLen(strCurrency) != 3)
   {
      Print("Problem with strCurrency = ''",strCurrency,"'' in ",strNewsFileName,". Should be 3 characters");
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // DateTime
   string strDateTimeFromFile = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 && nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      Print(ErrorDescription(nLastError)," reading DateTime from ",strNewsFileName);
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }
   
//////////////////////////////////////
   // Timezone
   string strTimeZone = FileReadString(handle); nLastError = GetLastError();
   if (nLastError != 0 &&  nLastError != ERR_NOT_INITIALIZED_STRING)
   {
      Print(ErrorDescription(nLastError)," reading TimeZone from ",strNewsFileName);
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }

//////////////////////////////////////
   // Convert DateTime
   datetime DateTimeFromFile = StrToTime(strDateTimeFromFile);
   if (DateTimeFromFile == 0)
   {
      Print("Problem with DateTime = ''",strDateTimeFromFile,"'' from ",strNewsFileName);
      if (bDebug) Print("Function ReadCSVline() returning false");
      return(false);
   }
   DateTimeServer = ServerTime(TimeToUTC(DateTimeFromFile, strTimeZone));

   if (bDebug) Print("Function ReadCSVline() returning true and (''",strDescription,"'',''",strCurrency,"'',",TimeToStr(DateTimeServer),")");
   return(true);
}
      
  
bool CurrencyPairContains(string strCurrency)
{
   bool bReturn = StringSubstr(Symbol(),0,3) == strCurrency || StringSubstr(Symbol(),3,3) == strCurrency;
   if (bDebug) Print("Function CurrencyPairContains(",strCurrency,") returning ",bToStr(bReturn));
   return(bReturn);
}

string bToStr(bool b)
{
   if (b) return("true"); 
   else return("false");
}

datetime LocalTimeDifference()
{
   int nRoundedMinutes = (LocalTime()-CurTime()+30)/60;
   return(nRoundedMinutes*60);
}